@@ -8,7 +8,7 @@
  * memorandum "A Kernel Model for Precision Timekeeping" by Dave Mills,
  * revised 31 January 1996. That document updates RFC1589.
  *
- * Copyright (c) 1996 - 1999 by Ulrich Windl
+ * Copyright (c) 1996, 1997, 1998 by Ulrich Windl
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,18 +24,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: enable_pps.c,v 1.7.2.2 1999/03/27 22:25:33 windl Exp $
- */
+ * $Id: enable_pps.c,v 1.8 1998/12/30 14:23:56 windl Exp $ */
 #include	<unistd.h>
 #include	<time.h>
 #include	<stdlib.h>
 #include	<stdio.h>
 #include	<string.h>
 #include	<errno.h>
-#include	<sys/utsname.h>
-
-#include	<linux/version.h>
-#define	NANO	/* prefer the Nanokernel if available */
 #include	<sys/timex.h>
 
 #include	<sys/ppsclock.h>
@@ -53,42 +48,10 @@
 				 */
 #endif
 
-#ifdef	STA_NANO		/* check if compiling with Nanokernel */
-#define	tv_frac	tv_nsec		/* using struct timespec */
-#else
-#define	tv_frac	tv_usec		/* using struct timeval */
-#endif
+#define OFFSET_TOLERANCE	5000
+#define STABIL_TOLERANCE	50
 
-#ifndef	SHIFT_USEC
-#define SHIFT_USEC	16	/* compatibility define */
-#define	PPS_SHIFT	4	/* compatibility define */
-#define	PPS_SHIFTMAX	8	/* compatibility define */
-#endif
-
-#ifndef	ADJ_ADJTIME
-#define ADJ_ADJTIME	ADJ_OFFSET_SINGLESHOT	/* compatibility define */
-#endif
-
-#define OFFSET_TOLERANCE	50000
-#define STABIL_TOLERANCE	(10 << SHIFT_USEC)
-#define	PPS_INSANE		(STA_PPSERROR|STA_PPSWANDER|STA_PPSJITTER)
-
-#define	ALL_BITS_ON(v, b)		(((v) & (b)) == (b))
-#define	ALL_BITS_OFF(v, b)		(((v) & (b)) == 0)
-#define	BITS_OFF(v, b)			((v) & ~(b))
-#define	BITS_ON(v, b)			((v) | (b))
-
-static	const char	id[] = "$Id: enable_pps.c,v 1.7.2.2 1999/03/27 22:25:33 windl Exp $";
-
-#define	BIT(n)	(1UL << (n))
-#define OPT_MONITOR		BIT(0)	/* perform monitor loop */
-#define OPT_RESET		BIT(1)	/* reset clock state */
-#define OPT_NO_OFFSET		BIT(2)	/* don't adjust offset */
-#define OPT_NO_STATUS		BIT(3)	/* don't adjust status */
-#define OPT_NO_TIMECONST	BIT(4)	/* don't adjust time constant */
-#define OPT_JITTER_LOOP		BIT(5)	/* monitor jitter only */
-
-static	unsigned long	options = OPT_MONITOR | OPT_RESET;
+static	const char	id[] = "$Id: enable_pps.c,v 1.8 1998/12/30 14:23:56 windl Exp $";
 
 static	char	polarity	= 'p';	/* p[ositive] | n[egative] | 0) */
 
@@ -107,7 +70,6 @@ static	void	print_header(void)
 	       "errcnt", "stbcnt");
 }
 
-/* (partially) print current time state */
 static	void	print_stats(const struct timex *tp)
 {
 	printf("%04x %7ld %7.3f %7.3f %4ld %1d %7.3f %4ld %4ld %4ld %4ld\n",
@@ -140,9 +102,9 @@ static	char const	*status_str[] = {	/* from <timex.h> */
 	"PPSERROR",	/* bit 11 */
 
 	"CLOCKERR",	/* bit 12 */
-	"NANO",		/* bit 13 */
-	"MODE",		/* bit 14 */
-	"CLK",		/* bit 15 */
+	"???13",	/* bit 13 */
+	"???14",	/* bit 14 */
+	"???15",	/* bit 15 */
 };
 
 /* write strings for set status bits on ``fp'' */
@@ -153,16 +115,16 @@ static	void	write_status_bits(FILE *fp, int bits)
 	for ( b = 0; b < 16; ++b )
 	{
 		if ( (bits & 1) != 0 )
-			fprintf(fp, "%s%s", sep, status_str[b]), sep = "+";
+			fprintf(fp, "%s%s", sep, status_str[b]), sep = ", ";
 		bits >>= 1;
 	}
 }
 
 /* print PPS status and update *evp */
-static	int	pps_status(const char *tag, const int show_event,
-			   int retval, int status, struct ppsclockev *evp)
+static	int	pps_status(int retval, int status, struct ppsclockev *evp)
 {
 	static	struct ppsclockev	last;
+	static	char const	tag[] = "PPS: ";
 	int	result = -1;
 	char const	*state_str = NULL;
 	char const	*sep	= tag;
@@ -173,7 +135,7 @@ static	int	pps_status(const char *tag, const int show_event,
 		memset(evp, 0, sizeof(*evp));
 	}
 	
-	if ( show_event && evp->serial != last.serial )
+	if ( evp->serial != last.serial )
 	{
 		long	secs	= evp->tv.tv_sec - last.tv.tv_sec;
 		long	evs	= evp->serial - last.serial;
@@ -201,12 +163,12 @@ static	int	pps_status(const char *tag, const int show_event,
 	if ( state_str != NULL )
 		fprintf(stderr, "%sstate=%s", sep, state_str), sep = ", ";
 
-	if ( evp->serial == 0 || !(status & STA_PPSSIGNAL) )
+	if ( !(status & STA_PPSSIGNAL) )
 		fprintf(stderr, "%sno pulse", sep), sep = ", ";
-	status &= (STA_PPSJITTER|STA_PPSERROR|STA_PPSWANDER|STA_CLOCKERR);
+	status &= (STA_PPSJITTER|STA_PPSERROR|STA_PPSWANDER);
 	if ( status != 0 )
 	{
-		fprintf(stderr, "%serrors=", sep), sep = ", ";
+		fprintf(stderr, "%s", sep), sep = ", ";
 		write_status_bits(stderr, status);
 	}
 	if ( sep != tag )
@@ -215,13 +177,12 @@ static	int	pps_status(const char *tag, const int show_event,
 }
 
 /* do adjtimex and check for errors */
-static	int	verbose_adjtimex(const char *tag, const int show_event,
-				 struct timex *txp, struct ppsclockev *evp)
+static	int	verbose_adjtimex(struct timex *txp, struct ppsclockev *evp)
 {
 	int	state = adjtimex(txp);
 	if ( state > 0 )
 	{
-		pps_status(tag, show_event, state, txp->status, evp);
+		pps_status(state, txp->status, evp);
 	}
 	else if ( state < 0 )
 	{
@@ -239,8 +200,6 @@ static	int	update_status(struct timex *txp, struct ppsclockev *evp,
 	int	change;
 	char	*sep = " (";
 
-	if ( options & OPT_NO_STATUS )
-		return(0);
 	txp->status |= set_bits;
 	txp->status &= ~clear_bits;
 	txp->modes = MOD_STATUS;
@@ -261,40 +220,21 @@ static	int	update_status(struct timex *txp, struct ppsclockev *evp,
 		fprintf(stderr, "]");
 	}
 	fprintf(stderr, "%s\n", sep[0] == ',' ? ")" : "");
-	return( verbose_adjtimex("update_status(): ", 0, txp, evp) );
+	return( verbose_adjtimex(txp, evp) );
 }
 
 /* reset kernel PLL */
 static	void	restart(struct timex *txp, struct ppsclockev *evp)
 {
-	if ( ALL_BITS_OFF(options, OPT_RESET) )
-		return;
-	txp->modes = MOD_ESTERROR | MOD_MAXERROR;
-	/* indicate that we are trying to synchronize */
-	txp->esterror = txp->maxerror = MAXPHASE / 1000;
-	txp->modes |= MOD_FREQUENCY | ADJ_TICKADJ;
+	txp->modes =
+		MOD_ESTERROR | MOD_MAXERROR | MOD_FREQUENCY | ADJ_TICKADJ;
+	txp->esterror = txp->maxerror = 123;	/* just some dummy value */
 	txp->freq = 0;
-	/* boost adjtime() times 100 (HZ*500 == 0.05s per second) */
-	txp->tickadj = 500;			
-	txp->modes |= MOD_TIMECONST;
-	txp->constant = 1;
-	verbose_adjtimex("restart(): ", 0, txp, evp);
-	update_status(txp, evp, STA_FLL | STA_PPSFREQ | STA_PPSTIME,
-		      STA_FREQHOLD | STA_PLL | STA_UNSYNC);
-}
-
-/* use current time constant from PPS discipline if it is working properly */
-static	void	update_constant(struct timex *txp, struct ppsclockev *evp)
-{
-	if ( options & OPT_NO_TIMECONST )
-		return;
-	if ( (txp->status & STA_PPSSIGNAL) != 0 &&
-	     (txp->status & PPS_INSANE) == 0 )
-		txp->constant = txp->shift;		/* copy PPS PLL constant */
-	else
-		txp->constant = 1;
-	txp->modes = MOD_TIMECONST;
-	verbose_adjtimex("update_constant(): ", 0, txp, evp);
+	txp->tickadj = 500;			/* boost adjtime() times 100 */
+	verbose_adjtimex(txp, evp);
+	update_status(txp, evp, STA_FLL | STA_PPSFREQ | STA_PPSTIME |
+		      STA_UNSYNC,
+		      STA_FREQHOLD | STA_PLL);
 }
 
 /* do offset adjustments. expects offset within bounds */
@@ -304,7 +244,6 @@ static	void	pll_adj_offset(struct timex *txp, struct ppsclockev *evp)
 	static	u_int  last_serial;
 	time_t	t = time(NULL);
 	long	offset = evp->tv.tv_usec;
-	long	tdelta, tconst;
 
 	if ( last_serial == evp->serial )
 		return;
@@ -315,34 +254,18 @@ static	void	pll_adj_offset(struct timex *txp, struct ppsclockev *evp)
 	txp->modes = MOD_MAXERROR | MOD_ESTERROR;
 	txp->esterror = txp->maxerror = offset >= 0 ? : -offset;
 	txp->offset = (txp->offset + offset) / 2;
-	if ( txp->offset > MAXPHASE / 2 || txp->offset < -MAXPHASE / 2 )
-		txp->modes = ADJ_ADJTIME;
+	if ( txp->offset > MAXPHASE / 2 && txp->offset < -MAXPHASE / 2 )
+		txp->modes = ADJ_OFFSET_SINGLESHOT;
 	else
 		txp->modes |= MOD_OFFSET;
-	tdelta = t - last_adj_time;
-	tconst = ALL_BITS_ON(txp->status, STA_PPSTIME|STA_PPSSIGNAL) ?
-		txp->shift : txp->constant;
-	if ( ((txp->status & (STA_FLL|STA_PPSFREQ)) != 0 &&
-	      (tdelta < MINSEC || tdelta < (1L << tconst))) ||
-	     ((txp->status & STA_PLL) != 0 && tdelta < (1L << tconst) / 2) )
+	if ( ((txp->status & STA_FLL) != 0 && t - last_adj_time < MINSEC) ||
+	     ((txp->status & STA_PLL) != 0 &&
+	      t - last_adj_time < (1L << txp->constant)) )
 		return;			/* don't adjust too frequently */
 	else
 		last_adj_time = t;
-	printf("%s: modes=%04x, delta=%ld, const=%ld, in=%ld, out=%ld\n",
-	       "pll_adj_offset", txp->modes, tdelta, tconst, offset,
-	       txp->offset);
-	verbose_adjtimex("pll_adj_offset(): ", 0, txp, evp);
-}
-
-/* adjust offset if conditions are sane */
-static	void	update_offset(struct timex *txp, struct ppsclockev *evp)
-{
-	if ( options & OPT_NO_OFFSET )
-		return;
-	if ( ((txp->status & STA_PPSSIGNAL) != 0 &&
-	      (txp->status & PPS_INSANE) == 0) ||
-	     (txp->status & STA_PPSSIGNAL) == 0 )
-		pll_adj_offset(txp, evp);	/* uses pps_offset! */
+printf("pll_adj_offset: in=%ld, out=%ld\n", offset, txp->offset);
+	verbose_adjtimex(txp, evp);
 }
 
 /* print status message */
@@ -351,191 +274,42 @@ static	void	status(char const *str)
 	fprintf(stderr, "[%s]\n", str);
 }
 
-/* execute calibration loop */
-static	void	calibrate(struct timex *txp, struct ppsclockev *evp)
-{
-	int		time_state;
-	int		state;
-	state = S_NO_PPS;
-	while ( 1 )
-	{
-		txp->modes = 0;
-		time_state = verbose_adjtimex("monitor: ", 1, txp, evp);
-		print_stats(txp);
-		switch ( state )
-		{
-		case S_NO_PPS:	/* wait for good PPS */
-			update_status(txp, evp,
-				      STA_PPSTIME | STA_PPSFREQ | STA_FLL,
-				      STA_PLL | STA_FREQHOLD);
-			update_offset(txp, evp);
-			if ( (txp->status & STA_PPSSIGNAL) != 0
-			     && (txp->status & PPS_INSANE) == 0
-			     && (txp->offset > -OFFSET_TOLERANCE
-				 && txp->offset < OFFSET_TOLERANCE) )
-			{
-  				state = S_PPS_TIME;
-				status("S_NO_PPS: PPS detected and in range");
-			}
-			break;
-		case S_PPS_TIME:	/* adjust the offset */
-			update_status(txp, evp,
-				      STA_FLL | STA_PPSFREQ,
-				      STA_PPSTIME | STA_FREQHOLD | STA_PLL);
-			update_offset(txp, evp);
-			if ( (evp->tv.tv_usec > 2 * OFFSET_TOLERANCE &&
-			      evp->tv.tv_usec <
-			      1000000 - 2 * OFFSET_TOLERANCE) )
-			{
-				status("S_PPS_TIME: PPS got insane");
-				state = S_NO_PPS;
-			}
-			else if ( (txp->status & STA_PPSSIGNAL) != 0 &&
-				  (txp->status & PPS_INSANE) == 0 &&
-				  txp->offset > -OFFSET_TOLERANCE / 2 &&
-				  txp->offset <= OFFSET_TOLERANCE / 2 &&
-				  txp->stabil <= STABIL_TOLERANCE &&
-				  txp->shift > PPS_SHIFT )
-			{
-  				state = S_PPS_FREQ;
-				status("S_PPS_TIME: PPS offset is low and stable");
-			}
-			break;
-		case S_PPS_FREQ:	/* adjust PLL frequency from PPS */
-			update_status(txp, evp,
-				      STA_FLL | STA_FREQHOLD,
-				      STA_PPSTIME | STA_PPSFREQ | STA_PLL);
-			update_constant(txp, evp);
-			update_offset(txp, evp);
-			if ( txp->shift == PPS_SHIFT ||
-			     txp->offset < -OFFSET_TOLERANCE ||
-			     txp->offset >= OFFSET_TOLERANCE ||
-			     (txp->status & STA_PPSSIGNAL) == 0 ||
-			     (txp->status & ~STA_PPSJITTER & PPS_INSANE) != 0 )
-			{
-				state = S_NO_PPS;
-				status("S_PPS_FREQ: insane PPS signal");
-			}
-			else if ( txp->shift == PPS_SHIFTMAX &&
-				  txp->offset > -OFFSET_TOLERANCE / 4 &&
-				  txp->offset <= OFFSET_TOLERANCE / 4 &&
-				  txp->stabil <= STABIL_TOLERANCE / 2 )
-			{
-				status("PPS_FREQ: best quality PPS signal");
-				state = S_PPS_FLL;
-			}
-			break;
-		case S_PPS_FLL:	/* adjust PLL frequency from PPS */
-			update_status(txp, evp,
-				      STA_FREQHOLD,
-				      STA_PPSTIME | STA_PPSFREQ | STA_FLL |
-				      STA_PLL);
-			update_constant(txp, evp);
-			if ( (txp->status & (STA_PPSSIGNAL)) == 0 ||
-			     (txp->status & ~STA_PPSJITTER & PPS_INSANE) != 0 ||
-			     txp->stabil > STABIL_TOLERANCE ||
- 			     txp->shift < PPS_SHIFTMAX )
-			{
-				state = S_PPS_FREQ;
-				status("S_PPS_FLL: PPS signal lost quality");
-			}
-			break;
-		}
-		sleep(1);
-	}
-}
-
-/* monitor PPS jitter */
-static	void	monitor_jitter(struct timex *txp, struct ppsclockev *evp)
-{
-	struct timeval	now, last;
-	long		delta, last_delta;
-	int		last_serial;
-	struct timespec	sleeper;
-
-	last_delta = last.tv_sec = last.tv_usec = last_serial = 0;
-	txp->modes = 0;
-	sleeper.tv_sec = 0;
-	sleeper.tv_nsec = 500000000;
-	while ( 1 )
-	{
-		pps_status("monitor_jitter(): ", 0, 0, txp->status, evp);
-		if ( evp->serial == last_serial )
-		{
-			nanosleep(&sleeper, &sleeper);
-			continue;
-		}
-		last_serial = evp->serial;
-		now = evp->tv;
-		delta =  now.tv_usec - last.tv_usec;
-		delta += (now.tv_sec - last.tv_sec) * 1000000L;
-		printf("%6d %8ld", evp->serial, delta - last_delta);
-		if ( delta - last_delta > 1000 || delta - last_delta < -1000 )
-		{
-			printf("\t%d:%ld %d:%ld",
-			       last.tv_sec, last.tv_usec,
-			       now.tv_sec, now.tv_usec);
-		}
-		printf("\n");
-		fflush(stdout);
-		last = now;
-		last_delta = delta;
-	}
-}
-
 /* provide a usage message */
 static	void	usage(void)
 {
 	fprintf(stderr,
 		"Known options are:\n"
-		"\t-C\tdon't adjust time constant in monitoring loop\n"
 		"\t-e\texit - do not enter monitoring loop\n"
-		"\t-j\tmonitor jitter only\n"
 		"\t-k\tkeep settings (when entering monitoring loop)\n"
-		"\t-O\tdon't adjust offset in monitoring loop\n"
 		"\t-p{p|0|n}\tpolarity (of pulse) positive|none|negative\n"
-		"\t-S\tdon't adjust status in monitoring loop\n"
 		);
 }
 
 /* input has to be redirected to the desired device! */
 int	main(int argc, char *argv[])
 {
-	struct utsname	un;
 	int		stat = TIOCM_RTS;	/* RTS MODEM control line */
 	struct serial_struct ss;
 	struct timex 	tx;
 	struct ppsclockev event;
+	int		time_state;
+	int		state;
 	int		ch;
+	int		keep_settings = 0;	/* keep kernel settings */
+	int		monitor = 1;		/* do monitoring loop */
 
 	fprintf(stderr,
 		"WARNING: This program may invalidate your system time!\n");
-	printf("(This version has been compiled in Linux %s with"
-#ifndef	STA_NANO
-	       "out"
-#endif
-	       " nanokernel support.\n", UTS_RELEASE);
-	uname(&un);
-	printf(" Now running %s %s)\n", un.sysname, un.release);
-	while ( (ch = getopt(argc, argv, "CejkOp:S")) != -1 )
+	while ( (ch = getopt(argc, argv, "ekp:")) != -1 )
 	{
 		switch ( ch )
 		{
-		case 'C':	/* don't adjust time constant */
-			options = BITS_ON(options, OPT_NO_TIMECONST);
-			break;
-		case 'e':	/* exit - don't enter monitoring loop */
-			options = BITS_OFF(options, OPT_MONITOR);
-			break;
-		case 'j':	/* monitor jitter only */
-			options = BITS_ON(options, OPT_JITTER_LOOP);
-			break;
 		case 'k':	/* keep settings - don't restart when entering
 				   monitoring loop */
-			options = BITS_OFF(options, OPT_RESET);
+			keep_settings = 1;
 			break;
-		case 'O':	/* don't adjust offset */
-			options = BITS_ON(options, OPT_NO_OFFSET);
+		case 'e':	/* exit - don't enter monitoring loop */
+			monitor = 0;
 			break;
 		case 'p':	/* select desired edge of the pulse */
 			polarity = optarg[0];
@@ -552,9 +326,6 @@ int	main(int argc, char *argv[])
 				polarity = '0';
 			}
 			break;
-		case 'S':	/* don't adjust time status */
-			options = BITS_ON(options, OPT_NO_STATUS);
-			break;
 		case '?':
 			fprintf (stderr, "Unknown option `-%c'.\n", optopt);
 			usage();
@@ -590,16 +361,94 @@ int	main(int argc, char *argv[])
 		polarity);
 	tx.modes = 0;
 	print_header();
-	verbose_adjtimex("init: ", 1, &tx, &event);
+	verbose_adjtimex(&tx, &event);
 	print_stats(&tx);
-	if ( options & OPT_RESET )
+	if ( !keep_settings )
 	{
 		restart(&tx, &event);
 		print_stats(&tx);
 	}
-	if ( options & OPT_JITTER_LOOP )
-		monitor_jitter(&tx, &event);
-	if ( options & OPT_MONITOR )
-		calibrate(&tx, &event);
+	state = S_NO_PPS;
+	while ( monitor )
+	{
+		if ( (time_state = verbose_adjtimex(&tx, &event)) == 0 )
+			pps_status(time_state, tx.status, &event);
+		print_stats(&tx);
+		switch ( state )
+		{
+		case S_NO_PPS:	/* wait for good PPS */
+			update_status(&tx, &event,
+				      STA_PPSTIME | STA_PPSFREQ | STA_FLL,
+				      STA_PLL | STA_FREQHOLD);
+			if ( time_state != 0 )
+				break;
+			else if ( (tx.status & STA_PPSSIGNAL) != 0
+				  && (tx.status &
+				      (STA_PPSJITTER|STA_PPSERROR)) == 0
+				  && (tx.offset > -OFFSET_TOLERANCE
+				      && tx.offset < OFFSET_TOLERANCE) )
+			{
+  				state = S_PPS_TIME;
+				status("S_NO_PPS: PPS detected and in range");
+			}
+			if ( (tx.status & STA_PPSSIGNAL) != 0 )
+			{
+				pll_adj_offset(&tx, &event);
+			}
+			break;
+		case S_PPS_TIME:	/* adjust the offset */
+			update_status(&tx, &event,
+				      STA_FLL | STA_PPSFREQ,
+				      STA_PPSTIME | STA_FREQHOLD | STA_PLL);
+			pll_adj_offset(&tx, &event);	/* uses pps_offset! */
+			if ( tx.stabil > (STABIL_TOLERANCE << SHIFT_USEC) ||
+			     (event.tv.tv_usec > 2 * OFFSET_TOLERANCE &&
+			      event.tv.tv_usec <
+			      1000000 - 2 * OFFSET_TOLERANCE) )
+			{
+				status("S_PPS_TIME: PPS got insane");
+				state = S_NO_PPS;
+			}
+			if ( time_state == 0 &&
+			     tx.offset > -OFFSET_TOLERANCE / 2 &&
+			     tx.offset < OFFSET_TOLERANCE / 2 &&
+			     tx.shift > PPS_SHIFT )
+			{
+  				state = S_PPS_FREQ;
+				status("S_PPS_TIME: PPS offset is low and stable");
+			}
+			break;
+		case S_PPS_FREQ:	/* adjust PLL frequency from PPS */
+			update_status(&tx, &event,
+				      STA_FLL | STA_FREQHOLD,
+				      STA_PPSTIME | STA_PPSFREQ | STA_PLL);
+			if ( tx.shift == PPS_SHIFT ||
+			     tx.offset > -OFFSET_TOLERANCE / 2 &&
+			     tx.offset < OFFSET_TOLERANCE / 2 &&
+			     !(tx.status & STA_PPSSIGNAL) )
+			{
+				state = S_NO_PPS;
+				status("S_PPS_FREQ: invalid PPS signal");
+			}
+			if ( time_state == 0 && tx.shift == PPS_SHIFTMAX )
+			{
+				status("PPS_FREQ: best quality PPS signal");
+				state = S_PPS_FLL;
+			}
+			break;
+		case S_PPS_FLL:	/* adjust PLL frequency from PPS */
+			update_status(&tx, &event,
+				      STA_FREQHOLD,
+				      STA_PPSTIME | STA_PPSFREQ | STA_FLL |
+				      STA_PLL);
+			if ( tx.shift <= PPS_SHIFTMAX )
+			{
+				state = S_PPS_FREQ;
+				status("S_PPS_FLL: PPS signal lost quality");
+			}
+			break;
+		}
+		sleep(1);
+	}
 	return(0);
 }
