@@ -1,10 +1,10 @@
-/* $Id: gen_pps.c,v 1.1.1.1 1997/09/13 14:37:07 windl Exp $
+/* $Id: gen_pps.c,v 1.4 1998/02/10 10:36:07 windl Exp $
  *	Test program to create a PPS signal on RTS of serial port.
  *	Used to test NTP PPS code when no other PPS source is available.
  *	Linux 2.0 RTC driver is needed.
  *
  *	Copyright (C) 1996, Paul Gortmaker.
- *	Copyright (C) 1996, 1997 by Ulrich Windl.
+ *	Copyright (C) 1996, 1997, 1998 by Ulrich Windl.
  *
  *	Released under the GNU General Public License, version 2,
  *	included herein by reference.
@@ -24,7 +24,7 @@
 #include <fcntl.h>
 #include <errno.h>
 
-static	const char	id[] = "$Id: gen_pps.c,v 1.1.1.1 1997/09/13 14:37:07 windl Exp $";
+static	const char	id[] = "$Id: gen_pps.c,v 1.4 1998/02/10 10:36:07 windl Exp $";
 
 static	int	jitter		= 0;	/* amount of jitter */
 static	int	polarity	= 1;	/* polarity */
@@ -66,7 +66,7 @@ static	void	pulse(int port)
 }
 
 /* parse options and generate pulses */
-void main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
 
 	int			fd, retval;
